#!/usr/bin/python3

import sys
import struct

from adt import load_adt
from utils import *
from proxy import *
from proxyutils import *

iface = UartInterface()
p = M1N1Proxy(iface, debug=False)
bootstrap_port(iface, p)
u = ProxyUtils(p, heap_size = 128 * 1024 * 1024)

adt = load_adt(u.get_adt())
"""
with open("adt.txt", "w") as f:
    f.write(str(adt))
"""

BASE = 0x230850000

NAMES = {
        # All values valid
        0: "unk0",

        # Valid mask 0xa1071115. N.b. writing bad values wedges the display
        # controller
        1: "unk1"

        # Valid mask 0x7ff07ff. Each nibble is an 11-bit signed integer (yes,
        # really), bottom one controlling the brightness and the upper one not
        # having a clear affect
        2: "brightness",

        # Valid mask 0xffff for each
        3: "unk3",
        4: "unk4",

        # 0x8 enables the channel, 0x2 always set but maybe noop. Valid mask
        # 0x1df. Honoured regardless of format. Applies after swizzling.
        5: "mask_r",
        6: "mask_g",
        7: "mask_b",
        8: "mask_a",

        # N.b. writing bad values to 9,10,11 wedges the display controller

        # TODO: document formats. Valid mask 0x1fffb
        # bit 0x0008 - vertical flip
        # bit 0x0010 - horizontal flip
        # 0x5000 - BGRA 8-bit UNORM
        # 0x5220 - A2B10G10R10 UNORM
        # 0x5280 - A2R10G10B10 UNORM
        # 0x50A0 - R8 UNORM, with some sort of overlay enabled
        # 0x5200 - no output
        # 0x5400 - no output
        # 0x5800 - no output
        # 0x5020 - like A2B10G10R10 but cropped and blinking and broken
        # 0x5040 - like A2B10G10R10 but cropped and blinking and broken
        # 0x5001 - wedge
        # 0x5002 - no output
        # 0x5003 - wedge
        # 0x5080 - like unknown???, cropped and blinking and broken
        # 0x5c80 - scewed up reading way out of bounds
        # 0x5100 - totally screwed up all over the place
        # 0x5180 - inconsistently screwed up!
        # 0x5a00 - double double toil and trouble
        # 0x4000 - no output
        # 0x7000 - no output
        # 0x6000 - no output
        # 0xf000 - no output
        # 0x0000 - no output
        # 0x1000 - like 0x5000 maybe?
        # 0x15220 - like 0x5220 but with an overlay and swizzled screwed up
        12: "format",

        # 4 bits for each channel, valid mask 0x3333. Common values include
        # 0x0123 for RGBA and 0x2103 for BGRA. Bottom 2 bits is alpha swizzle,
        # then blue, then green, then top two bits is red swizzle.  The swizzle
        # SHOULD be a true permutation, as internally the hardware applies the
        # inverse swizzle, and so behaviour is undefined if it's not a
        # permutation.
        #
        # Honoured for RGBA8. Ignored for 10-bit formats. (Possibly applies
        # bytewise instead of channels, so 10-bit wouldn't make sense)
        13: "swizzle",

        # Valid mask 0x1f1f1f, more pixel format stuff. Bottom byte controls
        # the plane 1, second byte controls plane 2, that implies there is a
        # plane 3 as well. The bottom 2 bits control something coloury. The
        # upper 3 bits control a number of pixels in a row to reverse the order
        # of (0 does no reversing, 7 flips all the characters in mini's tty,
        # vary in between for more effects)
        14: "unk14",

        # Valid mask 0x7fffffff. No visible effect.
        15: "unk15",
        16: "unk16",
        17: "unk17",

        # Valid mask 0xf. No visible effect.
        18: "unk18",
        19: "unk19",
        20: "unk20",

        # Display controller pointer to start of framebuffer. Note the display
        # controller is behind a DART and addresses resolved with the DART
        21: "framebuffer0",
        22: "framebuffer1",
        23: "framebuffer2",

        """
        Valid masks in bulk, none of these registers have visible effect
        24 0xf
        25 0xf
        26 0xf

        27 0xffffff80
        28 0xffffff80
        29 0xffffff80

        30 0xf
        31 0xf
        32 0xf

        33 0xffffff80
        34 0xffffff80
        35 0xffffff80
        
        36 0xf
        37 0xf
        38 0xf

        39 0xffffff80
        40 0xffffff80
        41 0xffffff80
        """

        # In units of bytes (linear framebuffer)
        42: "stride1",
        43: "stride2",
        44: "stride3",

        # Vertical offset up, reasonable values in [0, height). Affects all planes. Maybe useful for scrolling
        45: "y_offset_up",
        
        # Vertical offset, reasonable values in [0, height). Affects all planes. When used together with y_offset_up, acts as a crop.
        51: "y_offset_down",

        # Height in lower half, width in upper half
        52: "dimensions",
}

for i in range(0x1000):
    val = (p.read32(0x231344000 + i * 4))
    if val != 0:
        print("Block 3", i, hex(val))

for i in range(0x1000):
    val = (p.read32(0x231800000 + i * 4))
    if val != 0:
        print("Block 4", i, hex(val))

for i in range(0x1000):
    val = (p.read32(0x23B3D0000 + i * 4))
    if val != 0:
        print("Block 5", i, hex(val))

for i in range(0x400):
    val = (p.read32(0x23B738000 + i * 4))
    if val != 0:
        print("Block 6", i, hex(val))

for i in range(0x400):
    val = (p.read32(0x23BC3C000 + i * 4))
    if val != 0:
        print("Block 7", i, hex(val))



def reg_name(i):
    return NAMES[i].upper() if i in NAMES else i

for i in range(1024):
    val = p.read32(BASE + (i * 4))
    if val != 0:
        print("Block 0", reg_name(i), hex(val))

# Try to set the resolution. Note this is just cropping stuff it seems
def set_res(width, height):
    assert(width > 0 and width < 65536)
    assert(height > 0 and height < 65536)

    res = height | (width << 16)
    p.write32(BASE + (43 * 4), width)
    for i in [48, 49, 52, 70, 74]:
        p.write32(BASE + (i * 4), res)

set_res(1920, 1080)

fb = 0x9e0df0000
